#ifndef FABATHOMEPRINTER_H
#define FABATHOMEPRINTER_H

#include "Axis.h"
#include "Model.h"
#include "Motor.h"
#include "Path.h"
#include "Tool.h"

class FabAtHomePrinter
{
     //CONSTANTS/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
     private:
		 ///See JKerr's Documentation for more details on these variables
          byte X_Y_Z_GROUP_ADDRESS; //address of the controller board
          unsigned int NUM_MODULES; //number of moudles connected to the board
		  unsigned int BAUD_RATE; //rate of commuincation 
          unsigned int COM_PORT; //port that module is connected to (should be a serial port, use port numbers 0-9, set manually through hardware settings)
          double PLATFORM_DELTA; //speed the platform should move at
          double PRINT_ACCELERATION; //The acceleration that is used while printing.  The unit of PRINT_ACCELERATION is U/(second^2).
          double OLD_MSPS; //Magic constant from line 58 of FabAtHomeParameters.h in model 1 code.  Used to calculate values for pushout and suckback.
     //////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
     private:
          Model model; //The model that is to be printed
          map<string, Motor, LessThanString> motors; //A map from motor names to motors.
          map<string, MaterialCalibration, LessThanString> materialCalibrations; //A map from material calibration names to material calibrations.
          bool pausePrintFlag; //true when a pause in the printed is desired, false otherwise
		  bool cancelPrintFlag; //true when a print cancel has been requrested, false otherwise
		  bool redoPathFlag; //true when the current path is flagged to repeat, false otherwise
		  bool printing; //true when printing is in progress, false otherwise
		  bool initialized; //true when printer is inialized with a config file, false otherwise
		  bool paused; //true when printing is paused, false otherwise

          //Any \ in filePath must be escaped with \
          //Returns: "" iff successful or an error message.
          string loadConfigFile(string filePath);

          //Switch printer to path mode.
          //Returns: True iff successful.
          bool initializePathMode(void);         

          //Execute the path.
          //The bay will execute pushout, standard flow, and suckback iff flowDuringPath is true.
          //A call to this function returns when movement has finished.
          void executePath(Path& path, Bay* bay, const bool flowDuringPath);

          //Execute the points.
          //The bay will execute pushout, standard flow, and suckback iff flowDuringPath is true.
          //A call to this function returns when all points are in the path point buffer.
          void executePoints(const vector<Point>& points, Bay* bay, const bool flowDuringPath, double pushoutDistance, double pushoutVelocity, double standardDistance, double standardVelocity, double suckbackDistance, double suckbackVelocity, const Point& suckbackPoint);

          //Move all points from segment list to path point buffer.  Physical movement begins as soon as a point is in the path point buffer.
          //The bay will execute pushout, standard flow, and suckback iff flowDuringPath is true.
          void downloadSegmentList(Bay* bay, bool startFlow, bool startSuckback, double pushoutDistance, double pushoutVelocity, double standardDistance, double standardVelocity, double suckbackDistance, double suckbackVelocity, const Point& suckbackPoint);

          //Clears the segment list and then tries to add points into the segment list starting from startIndex.
          //Segment list has size 1000 but this function does not necessarily fill it completely.
          //Returns: The last index of points that was added to the segment list or -1 if points is empty.	
          int fillSegmentList(const vector<Point>& points, const int startIndex);

          //Returns: A bay that has the material calibration loaded or NULL if no such bay exists.
          Bay* findBay(MaterialCalibration*);

		  //Executes a setup path to the given point
          void executePausePath(const Point&);

		  //Executes a setup path from the first point to the secound point
          void executeSetupPath(const Point&, const Point&, const double&);
                    
		  //Loads a material collabration from the xml node
          string loadMaterialCalibration(TiXmlNode*);

		  //Loads a path from the xml node
          string loadPath(TiXmlNode*);

          //The copy constructor.  Private to prevent printer from being copied.
          FabAtHomePrinter(const FabAtHomePrinter&);

          //The copy assignment operator.  Private to prevent printer from being copied.
          FabAtHomePrinter& operator=(const FabAtHomePrinter&);
          
     public:
          map<string, Axis, LessThanString> axes; //A map from axis names to axes.
          Tool tool; //The given tool

          FabAtHomePrinter(void); //blank contrsuctor

          //Inserts into result the names of all material calibrations that are currently loaded.
          void loadedMaterialCalibrations(vector<string>& result);

          //Returns: The number of paths currently loaded.
          int numLoadedPaths(void);

          //Returns: The number of bays currently loaded.
          int numLoadedBays(void);

          //Returns: "PRINTING", "PAUSED", or "IDLE" depending on what printer is currently doing.
          string state(void);

          //Pause print after the current path finishes.
          void pausePrint(void);

          //Cancel print after the current path finishes.
          void cancelPrint(void);

          //Redo the current path after the current path finishes.
          void redoPath(void);

          //Any \ in filePath must be escaped with \
          //Load a file specifying material configurations and paths.
          //Returns: "" iff successful or an error message.
          string loadFabFile(string filePath);

		//save config filePath
		  static void savePreviousConfigName(string name);

		  //load config filePath
		  static string loadPreviousConfigName();

          //Initialize the printer.
          //Returns: "" if successful or an error message if failed.
          string initialize(const string& configFilePath);

          //Equip a bay with a material calibration or unequip the bay if the material calibration is "".
          //Requires: !isPrinting() || isPaused() and the bay and material calibration have been loaded.
          void equipBay(const string& bayName, const string& materialCalibrationName);


          //Execute all paths which are currently loaded.
          //A call to this function returns when the print has finished.
          //If fabricationThread != NULL, messages will periodically be stored in displayText
          //describing what the printer is currently doing and progress will be reported to fabricationThread when the message changes.
          void print(System::ComponentModel::BackgroundWorker* fabricationThread, string& displayText);

          //Perform clean up when done using printer.
          //Returns: True iff successful.
          bool cleanUp(void);

          //The destructor.
          ~FabAtHomePrinter(void);
};

#endif //ndef FABATHOMEPRINTER_H