#ifndef MOTOR_H
#define MOTOR_H

#include "util/Util.h"
using namespace System::Runtime::InteropServices; //Needed for DLLs
//PIC-SERVO base header files
#include "sio_util.h"
#include "nmccom.h"
#include "picservo.h"
#include "picio.h"
//PIC-SERVO path header files
#include "picServoPath.h"

class Motor
{
//CONSTANTS/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
private:
	string NAME;
	byte ADDRESS; //The number of the motor port that this motor is plugged in.
    double COUNTS_PER_U; //Number of counts for 1 U of motion.
	double TICKS_PER_SECOND; //Number of ticks that a motor makes per second.
    double MIN_COMMANDED_POSITION; //The minimum position that the motor can go to
	double MAX_COMMANDED_POSITION; //The maxiumum position that the motor can go to
	double MIN_COMMANDED_VELOCITY; //The minumum velocity the motor can travel at
	double MAX_COMMANDED_VELOCITY; //The maximum velocity the motor can travel at
	double MIN_COMMANDED_ACCELERATION; //The minimum acceleration of the motor
	double MAX_COMMANDED_ACCELERATION; //The maxiumum acceleration of the motor
	short KP, KD, KI, IL, EL; //Damping contraints (look up procedure for critically damping motors for more details, also JKerr has documentation)
	byte OL, CL, SR, DB; //See JKerr's Document for more details on his motor bytes
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
private:
     double commandedPosition; //desired position of the motor
     bool initialized; //true is motor is initialized and ready for us, false otherwise
     bool isReversed; //true if motor direction is to be reversed from default values on controller board

     //Tell hardware to send updated data for this motor to the DLL.
     void updateData(void);

public:
	Motor(void);

	//Creates a motor with all of the given values (described above)
	Motor(string name,const byte& address, const double countsPerU, const double ticksPerSecond, short kp, short kd, short ki, short il, byte ol, byte cl, short int el, byte sr, byte db);
	
	//Returns: The current position of the motor.  The unit of position is U.
	double getPosition(void);
	
	//Returns: The most recent position that the motor was commanded to move to by moveAbsolute, moveRelative, and resetPosition functions.  The unit of position is U.
	double getCommandedPosition(void);
	
	//Move to a position at a specified velocity and acceleration.
	//The unit of position is U, the unit of velocity is U/second, and the unit of acceleration is U/(second^2).
	//Requires MIN_COMMANDED_POSITION <= position <= MAX_COMMANDED_POSITION, MIN_COMMANDED_VELOCITY <= velocity <= MAX_COMMANDED_VELOCITY, MIN_COMMANDED_ACCELERATION <= acceleration <= MAX_COMMANDED_ACCELERATION
	//Returns true iff command was issued successfully.
	bool moveAbsolute(double position, double velocity, double acceleration);
	
	//Move relative to the current position at a specified velocity and acceleration.
	//Returns true iff command was issued successfully.
	bool moveRelative(double positionDelta, double velocity, double acceleration);

     //Returns: True iff motor is currently moving.
     bool moving(void);

	//This function returns as soon as possible after the motor has finished moving.
	void waitMove(void);
	
	//Stops the motor
	bool stop(void);
	
     //Set whether this motor is reversed.  If a motor is reversed, movement commands will cause the motor to move in the opposite direction.
     void setReversed(bool);

	 //sets the address of the motor on the controller board, see JKerr documentation for more details
	bool setGroup(byte groupAddress, bool isLeader);
	
     //The motors current position is considered to be 0 and the motors commanded position is set to 0.
	bool resetPosition(void);
	
	//Initialize the motor.
	//Returns: "" if successful or an error message if failed.
     string initialize(void);
	
	 //gets the motor address from the controller board
     byte getAddress(void);

	 //gets the ratio of counts of motor movement per real unit of measurement
     double getCountsPerU(void);

     //Perform cleanup when done using motor.
	//Returns: True iff successful.
	bool cleanUp(void);

     //The destructor.
     ~Motor(void);

};

#endif //ndef MOTOR_H